==> dynatrace_flutter_plugin-3.243.0.tar.gz: lib/src/cli/config/android_configurator.dart <==
import '../../agent/util/logger.dart';
import '../../cli/config/model/cli_arguments.dart';
import '../../cli/util/pathConstants.dart';
import '../../cli/config/model/android_config.dart';
import '../../cli/config/gradle_files.dart';

import 'dart:io';

/// Responsible for reading the configuration string which contains
/// all the gralde configuration values for the Android Agent
class AndroidConfigurator {
  /// Identifier for the configuration block in the gradle file
  static const String gradleConfigIdentifier = "// AUTO - INSERTED";

  /// String template for applying the dynatrace.gradle file within a gradle file
  static const String gradleDynatraceFile =
      "apply from: \"./dynatrace.gradle\"";

  /// String template for applying the plgin.gradle file within a gradle file
  static const String gradleApplyBuildscript =
      "apply from: \"./plugin.gradle\", to: buildscript";

  late PathConstants _paths;
  late File _buildGradleFile;

  /// Constructor which uses the [arguments] passed to the CLI and
  /// and optional [pListFile] as a fallback
  AndroidConfigurator(PathConstants paths, CommandLineArguments arguments,
      File? buildGradleFile) {
    _paths = paths;

    if (buildGradleFile == null) {
      _buildGradleFile = File(arguments.gradlePath!);
    } else {
      _buildGradleFile = buildGradleFile;
    }
  }

  /// Configuring the android folder more specificaly the gradle file which should be
  /// found under [_buildGradleFile].
  Future<void> instrumentAndroidPlatform() async {
    if (await _gradleFileExists(_buildGradleFile)) {
      await _changeFlutterBuildGradleFile();
    } else {
      throw const FileSystemException(
          "Can't find .gradle file. Gradle path must also include the gradle file!");
    }
  }

  /// Configures and modifies the Gradle file which can be found under [_customGradleFile].
  Future<void> _changeFlutterBuildGradleFile() async {
    List<String> gradleFileContentLines = await gradleAsList();

    int gradlePluginFileIndex = -1;
    int gradleDynatraceFileIndex = -1;

    for (int i = 0;
        i < gradleFileContentLines.length &&
            (gradleDynatraceFileIndex == -1 || gradlePluginFileIndex == -1);
        i++) {
      if (gradleFileContentLines[i].contains("plugin.gradle")) {
        gradlePluginFileIndex = i;
      } else if (gradleFileContentLines[i].contains("dynatrace.gradle")) {
        gradleDynatraceFileIndex = i;
      }
    }

    bool modified = false;

    if (gradlePluginFileIndex == -1) {
      int gradleFileFlutterIndex = -1;
      for (int i = 0; i < gradleFileContentLines.length; i++) {
        if (gradleFileContentLines[i].startsWith("buildscript")) {
          gradleFileFlutterIndex = i;
          break;
        }
      }

      if (gradleFileFlutterIndex == -1) {
        throw Exception("Could not find Buildscript block in build.gradle.");
      }

      gradleFileContentLines.insert(
          gradleFileFlutterIndex + 1, gradleApplyBuildscript);
      modified = true;
    }

    if (gradleDynatraceFileIndex == -1) {
      gradleFileContentLines.insert(0, gradleDynatraceFile);
      modified = true;
    }

    String fullGradleFile = gradleFileContentLines.join("\n");
    if (modified) {
      await _buildGradleFile.writeAsString(fullGradleFile);
      Logger().d(
          "Added Dynatrace plugin and agent to build.gradle: ${_buildGradleFile.path}");
    } else {
      Logger().d(
          "Dynatrace plugin and agent already added to build.gradle file: ${_buildGradleFile.path}");
    }
  }

  /// Modify the Gradle file and write it. The configuration which should be written is in [androidConfig].
  Future<void> writeGradleConfig(AndroidConfiguration androidConfig) async {
    if (androidConfig.getConfig() == null) {
      Logger().i(
          "Can't write configuration of Android agent because it is missing!",
          logType: LogType.Warning);
      return;
    }

    Directory gradleDir = _buildGradleFile.parent;

    if (!await gradleDir.exists()) {
      throw const FileSystemException(
          "Gradle Directory doesn't exist. Can't continue to write configuration.");
    }

    File dynatraceGradle = File(_paths.getDynatraceGradle(gradleDir));
    File pluginGradle = File(_paths.getPluginGradle(gradleDir));

    // Check if dynatrace.gradle exists - if not create it
    if (!await dynatraceGradle.exists()) {
      await dynatraceGradle.writeAsString(getGradleDynatraceContent());
    }

    // Always write the plugin.gradle as it is controlling the version of the plugin
    await pluginGradle.writeAsString(getGradlePluginContent());

    String gradleFileContent = await dynatraceGradle.readAsString();
    List<String?> gradleFileContentLines =
        _removeOldGradleConfig(gradleFileContent);

    int gradleFileIndex = -1;
    for (int i = 0; i < gradleFileContentLines.length; i++) {
      if (gradleFileContentLines[i]!.contains(gradleConfigIdentifier)) {
        gradleFileIndex = i;
        break;
      }
    }

    // Insert Gradle Command
    gradleFileContentLines.insert(
        gradleFileIndex + 1, androidConfig.getConfig());

    String fullGradleFile = gradleFileContentLines.join("\n");
    dynatraceGradle.writeAsString(fullGradleFile);

    Logger().d(
        "Replaced old configuration with current configuration in dynatrace.gradle");
  }

  /// Removing old gradle stuff that should not be used anymore.
  List<String?> _removeOldGradleConfig(String gradleFileContent) {
    List<String?> gradleFileContentLines = gradleFileContent.split("\n");

    List<int> gradleConfigIndex = [];
    for (int i = 0;
        i < gradleFileContentLines.length && gradleConfigIndex.length < 2;
        i++) {
      if (gradleFileContentLines[i]!.contains(gradleConfigIdentifier)) {
        gradleConfigIndex.add(i);
      }
    }

    if (gradleConfigIndex.length != 2) {
      throw Exception(
          "Could not find identfier in internal gradle file. Please re-install.");
    }

    gradleFileContentLines.removeRange(
        gradleConfigIndex[0] + 1, gradleConfigIndex[1]);
    return gradleFileContentLines;
  }

  /// Removing again all gradle trace in case of uninstalling the plugin
  Future<void> removeGradleLinesForUninstall() async {
    if (await _gradleFileExists(_buildGradleFile)) {
      // Remove dynatrace.gradle and plugin.gradle references from build.gradle
      List<String> gradleContent = await gradleAsList();

      if (gradleContent.contains(gradleApplyBuildscript)) {
        gradleContent.remove(gradleApplyBuildscript);
      }

      if (gradleContent.contains(gradleDynatraceFile)) {
        gradleContent.remove(gradleDynatraceFile);
      }

      String fullGradleFile = gradleContent.join("\n");
      await _buildGradleFile.writeAsString(fullGradleFile);
      Logger().d(
          "Removed dynatrace references from the gradle file: ${_buildGradleFile.path}");
    } else {
      throw const FileSystemException(
          "Can't find build.gradle file. Skipping removal of dynatrace references in gradle file as it does not exist!");
    }
  }

  /// Removing both dynatrace related gradle files (dynatrace.gradle and plugin.gradle)
  Future<void> removeDynatraceGradleFiles() async {
    File dynatraceGradle =
        File(_paths.getDynatraceGradle(_buildGradleFile.parent));
    File pluginGradle = File(_paths.getPluginGradle(_buildGradleFile.parent));
    // Remove dynatrace.gradle and plugin.gradle files
    try {
      if (await _gradleFileExists(dynatraceGradle)) {
        await dynatraceGradle.delete();
      }

      if (await _gradleFileExists(pluginGradle)) {
        await pluginGradle.delete();
      }

      Logger().d("Removed dynatrace gradle confguration files!",
          logType: LogType.Info);
    } catch (e) {
      Logger().i(
          "Unable to remove dynatrace gradle confguration files during uninstall: ${e.toString()}",
          logType: LogType.Warning);
    }
  }

  Future<bool> _gradleFileExists(File gradleFile) async {
    return gradleFile.path.endsWith(".gradle") && await gradleFile.exists();
  }

  /// Receive the configuration gradle string as a list
  Future<List<String>> gradleAsList() async {
    String gradleFileContent = await _buildGradleFile.readAsString();
    return gradleFileContent.split("\n");
  }
}

==> dynatrace_flutter_plugin-3.279.2.tar.gz: lib/src/cli/config/android_configurator.dart <==
import '../../agent/util/logger.dart';
import '../../cli/config/model/cli_arguments.dart';
import '../../cli/util/pathConstants.dart';
import '../../cli/config/model/android_config.dart';
import '../../cli/config/gradle_files.dart';

import 'dart:io';

/// Responsible for reading the configuration string which contains
/// all the gralde configuration values for the Android Agent
class AndroidConfigurator {
  /// Identifier for the configuration block in the gradle file
  static const String gradleConfigIdentifier = "// AUTO - INSERTED";

  /// String template for applying the dynatrace.gradle file within a gradle file
  static const String gradleDynatraceFile =
      "apply from: \"./dynatrace.gradle\"";

  /// String template for applying the plgin.gradle file within a gradle file
  static const String gradleApplyBuildscript =
      "apply from: \"./plugin.gradle\", to: buildscript";

  late PathConstants _paths;
  late File _buildGradleFile;

  /// Constructor which uses the [arguments] passed to the CLI and
  /// and optional [buildGradleFile] as a fallback
  AndroidConfigurator(PathConstants paths, CommandLineArguments arguments,
      File? buildGradleFile) {
    _paths = paths;

    if (buildGradleFile == null) {
      _buildGradleFile = File(arguments.gradlePath!);
    } else {
      _buildGradleFile = buildGradleFile;
    }
  }

  /// Configuring the android folder more specificaly the gradle file which should be
  /// found under [_buildGradleFile].
  Future<void> instrumentAndroidPlatform() async {
    if (await _gradleFileExists(_buildGradleFile)) {
      await _changeFlutterBuildGradleFile();
    } else {
      throw const FileSystemException(
          "Can't find .gradle file. Gradle path must also include the gradle file!");
    }
  }

  /// Configures and modifies the Gradle file which can be found under [_customGradleFile].
  Future<void> _changeFlutterBuildGradleFile() async {
    List<String> gradleFileContentLines = await gradleAsList();

    int gradlePluginFileIndex = -1;
    int gradleDynatraceFileIndex = -1;

    for (int i = 0;
        i < gradleFileContentLines.length &&
            (gradleDynatraceFileIndex == -1 || gradlePluginFileIndex == -1);
        i++) {
      if (gradleFileContentLines[i].contains("plugin.gradle")) {
        gradlePluginFileIndex = i;
      } else if (gradleFileContentLines[i].contains("dynatrace.gradle")) {
        gradleDynatraceFileIndex = i;
      }
    }

    bool modified = false;

    if (gradlePluginFileIndex == -1) {
      int gradleFileFlutterIndex = -1;
      for (int i = 0; i < gradleFileContentLines.length; i++) {
        if (gradleFileContentLines[i].startsWith("buildscript")) {
          gradleFileFlutterIndex = i;
          break;
        }
      }

      if (gradleFileFlutterIndex == -1) {
        throw Exception("Could not find Buildscript block in build.gradle.");
      }

      gradleFileContentLines.insert(
          gradleFileFlutterIndex + 1, gradleApplyBuildscript);
      modified = true;
    }

    if (gradleDynatraceFileIndex == -1) {
      gradleFileContentLines.insert(0, gradleDynatraceFile);
      modified = true;
    }

    String fullGradleFile = gradleFileContentLines.join("\n");
    if (modified) {
      await _buildGradleFile.writeAsString(fullGradleFile);
      Logger().d(
          "Added Dynatrace plugin and agent to build.gradle: ${_buildGradleFile.path}");
    } else {
      Logger().d(
          "Dynatrace plugin and agent already added to build.gradle file: ${_buildGradleFile.path}");
    }
  }

  /// Modify the Gradle file and write it. The configuration which should be written is in [androidConfig].
  Future<void> writeGradleConfig(AndroidConfiguration androidConfig) async {
    if (androidConfig.getConfig() == null) {
      Logger().i(
          "Can't write configuration of Android agent because it is missing!",
          logType: LogType.Warning);
      return;
    }

    Directory gradleDir = _buildGradleFile.parent;

    if (!await gradleDir.exists()) {
      throw const FileSystemException(
          "Gradle Directory doesn't exist. Can't continue to write configuration.");
    }

    File dynatraceGradle = File(_paths.getDynatraceGradle(gradleDir));
    File pluginGradle = File(_paths.getPluginGradle(gradleDir));

    // Check if dynatrace.gradle exists - if not create it
    if (!await dynatraceGradle.exists()) {
      await dynatraceGradle.writeAsString(getGradleDynatraceContent());
    }

    // Always write the plugin.gradle as it is controlling the version of the plugin
    await pluginGradle.writeAsString(getGradlePluginContent());

    String gradleFileContent = await dynatraceGradle.readAsString();
    List<String?> gradleFileContentLines =
        _removeOldGradleConfig(gradleFileContent);

    int gradleFileIndex = -1;
    for (int i = 0; i < gradleFileContentLines.length; i++) {
      if (gradleFileContentLines[i]!.contains(gradleConfigIdentifier)) {
        gradleFileIndex = i;
        break;
      }
    }

    // Insert Gradle Command
    gradleFileContentLines.insert(
        gradleFileIndex + 1, androidConfig.getConfig());

    String fullGradleFile = gradleFileContentLines.join("\n");
    dynatraceGradle.writeAsString(fullGradleFile);

    Logger().d(
        "Replaced old configuration with current configuration in dynatrace.gradle");
  }

  /// Removing old gradle stuff that should not be used anymore.
  List<String?> _removeOldGradleConfig(String gradleFileContent) {
    List<String?> gradleFileContentLines = gradleFileContent.split("\n");

    List<int> gradleConfigIndex = [];
    for (int i = 0;
        i < gradleFileContentLines.length && gradleConfigIndex.length < 2;
        i++) {
      if (gradleFileContentLines[i]!.contains(gradleConfigIdentifier)) {
        gradleConfigIndex.add(i);
      }
    }

    if (gradleConfigIndex.length != 2) {
      throw Exception(
          "Could not find identfier in internal gradle file. Please re-install.");
    }

    gradleFileContentLines.removeRange(
        gradleConfigIndex[0] + 1, gradleConfigIndex[1]);
    return gradleFileContentLines;
  }

  /// Removing again all gradle trace in case of uninstalling the plugin
  Future<void> removeGradleLinesForUninstall() async {
    if (await _gradleFileExists(_buildGradleFile)) {
      // Remove dynatrace.gradle and plugin.gradle references from build.gradle
      List<String> gradleContent = await gradleAsList();

      if (gradleContent.contains(gradleApplyBuildscript)) {
        gradleContent.remove(gradleApplyBuildscript);
      }

      if (gradleContent.contains(gradleDynatraceFile)) {
        gradleContent.remove(gradleDynatraceFile);
      }

      String fullGradleFile = gradleContent.join("\n");
      await _buildGradleFile.writeAsString(fullGradleFile);
      Logger().d(
          "Removed dynatrace references from the gradle file: ${_buildGradleFile.path}");
    } else {
      throw const FileSystemException(
          "Can't find build.gradle file. Skipping removal of dynatrace references in gradle file as it does not exist!");
    }
  }

  /// Removing both dynatrace related gradle files (dynatrace.gradle and plugin.gradle)
  Future<void> removeDynatraceGradleFiles() async {
    File dynatraceGradle =
        File(_paths.getDynatraceGradle(_buildGradleFile.parent));
    File pluginGradle = File(_paths.getPluginGradle(_buildGradleFile.parent));
    // Remove dynatrace.gradle and plugin.gradle files
    try {
      if (await _gradleFileExists(dynatraceGradle)) {
        await dynatraceGradle.delete();
      }

      if (await _gradleFileExists(pluginGradle)) {
        await pluginGradle.delete();
      }

      Logger().d("Removed dynatrace gradle confguration files!",
          logType: LogType.Info);
    } catch (e) {
      Logger().i(
          "Unable to remove dynatrace gradle confguration files during uninstall: ${e.toString()}",
          logType: LogType.Warning);
    }
  }

  Future<bool> _gradleFileExists(File gradleFile) async {
    return gradleFile.path.endsWith(".gradle") && await gradleFile.exists();
  }

  /// Receive the configuration gradle string as a list
  Future<List<String>> gradleAsList() async {
    String gradleFileContent = await _buildGradleFile.readAsString();
    return gradleFileContent.split("\n");
  }
}
